import numpy as np
import pylab
from lmfit import Minimizer, Parameters, fit_report

#load the data to mattrix data
data=np.loadtxt('Data.dat')  
xdata=data[:,0]  # data of wavenumber
ydata=data[:,1]  # reflectivity data

#theoretical calculation of reflectivity
def Rteor(params, x):
    v = params.valuesdict()    #rozbaleni parametru do slovniku v
    e = v['eINF']-v['w']**2/(x*(x+1j*v['g']))  #dielektricka funkce
    N = e**0.5      #index lomu
    R=abs((N-1)/(N+1))**2 #vypocet odrazivosti
    return R    

# definition of residual function : difference between theory and experiment
def residual(params, x, ydata):
	return Rteor(params, x)-ydata

#creatin of parameters and settin their starting values: in nonlinear fit should not be far from the end result. 
#it is possible to set whether the parameter is fitted or not via (vary=True, vary=False) 
StartParams = Parameters()
StartParams.add('eINF', value=9,vary=True)
StartParams.add('w', value=1000,vary=True)
StartParams.add('g', value=100,vary=True)
original= Rteor(StartParams,xdata)  #calculate original spectrum : in nonlinear fit should not be far from data 

#fitting
minner = Minimizer(residual, StartParams, fcn_args=(xdata,ydata))
result = minner.minimize(method='leastsq')
print(fit_report(result))  # write error report, alternative result.params.pretty_print()
FitParams=result.params  #slovnik s objekty Parameter s nafitovanymi hodnotami parametru a jejich chybami. 
final = Rteor(FitParams,xdata) # calculate final fitted spectrum 

#for following work with fitted parameters can be helpful:
print ()
print ("hodnota eINF: ",  FitParams['eINF'].value, " a jeho chyba",  FitParams['eINF'].stderr)
print ("hodnoty korelacnich koeficientu s dalsimi parametry: ", FitParams['eINF'].correl)

#ulozeni nafitovaneho spektra do souboru
FileData= open('Spektrum.dat', 'w')
for i in range(len(xdata)):
    print (xdata[i], "\t",final[i], file=FileData)

#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w')
print (fit_report(result), file=FileFit)

# vykresleni fitu na monitor
pylab.plot(xdata, original, 'g-', label='Model spectrum with start parameters')
pylab.plot(xdata, ydata, 'k-', label='data')
pylab.plot(xdata, final, 'r-', label='Model spectrum with fitted parameters')
legend = pylab.legend(loc='upper right', shadow=False, fontsize='x-small')
pylab.show()

